import sys
from functools import partial
from contextlib import closing
import asyncio

READ_SIZE = 20000

async def main(loop, source_port, dest_host, dest_port):
    connect_event = asyncio.Event()
    server_closed_event = asyncio.Event()
    bound_cb = partial(proxy,
                       loop, connect_event, server_closed_event,
                       dest_host, dest_port)
    server = await asyncio.start_server(bound_cb, "localhost", source_port,
                                        loop=loop)
    await connect_event.wait()
    server.close()
    await server.wait_closed()
    server_closed_event.set()

async def proxy(loop, connect_event, server_closed_event,
                dest_host, dest_port,
                source_reader, source_writer):
    connect_event.set()
    try:
        with closing(source_writer):
            tmp = await asyncio.open_connection(dest_host, dest_port, loop=loop)
            dest_reader, dest_writer = tmp
            with closing(dest_writer):
                await copy_all(source_reader, dest_writer)
    finally:
        await server_closed_event.wait()
        loop.stop()

async def copy_all(source_reader, dest_writer):
    while True:
        await dest_writer.drain()
        data = await source_reader.read(READ_SIZE)
        if not data:  # EOF
            return
        dest_writer.write(data)

if __name__ == "__main__":
    try:
        args = [int(sys.argv[1]), sys.argv[2], int(sys.argv[3])]
    except Exception:
        print("Usage: {} SOURCE_PORT DEST_HOST DEST_PORT".format(__file__))
    else:
        loop = asyncio.get_event_loop()
        loop.create_task(main(loop, *args))
        loop.run_forever()
